package common

import (
	"testing"
	"time"
)

func TestMetricsClean(t *testing.T) {
	t.Parallel()

	now := time.Now()

	m := MakeMetricData()
	m.TTL = time.Hour
	m.Measures["a"] = &Metric{
		Name:  "should",
		Value: 1,
		ts:    now,
	}
	m.Measures["b"] = &Metric{
		Name:  "should not",
		Value: 1,
		ts:    now.Add(-2 * time.Hour),
	}

	m.Clean()

	if len(m.Measures) > 1 {
		t.Fatalf("Expected 1 metric, got %d", len(m.Measures))
	}

	for _, metric := range m.Measures {
		if metric.Name != "should" {
			t.Fatalf("Expected metric's name to be 'should', got: %s", metric.Name)
		}
	}
}

func TestAccumulatorClean(t *testing.T) {
	t.Parallel()

	now := time.Now()

	acc := &Accumulator{
		ttl: time.Hour,
		values: []*accValue{
			{1, now.Add(-3 * time.Hour)},
			{1, now.Add(-2 * time.Hour)},
			{1, now.Add(-3 * time.Minute)},
			{1, now.Add(-2 * time.Minute)},
			{1, now},
		},
	}

	acc.clean()

	values := acc.values
	if len(values) != 3 {
		t.Fatalf("Expected 3 values in accumulator, got %d", len(values))
	}

	acc = &Accumulator{
		ttl: time.Hour,
		values: []*accValue{
			{1, now.Add(-3 * time.Hour)},
		},
	}

	acc.clean()

	values = acc.values
	if len(values) != 0 {
		t.Fatalf("Expected 0 values in accumulator, got %d", len(values))
	}
}

func TestEmptyAccumulatorValues(t *testing.T) {
	t.Parallel()

	now := time.Now()

	acc := &Accumulator{
		outputType: "stats",
		ttl:        time.Hour,
		values: []*accValue{
			{1, now.Add(-3 * time.Hour)},
		},
	}

	res := acc.Values()

	if len(res) != 1 {
		t.Fatalf("Expected 'Values()' length to be 1, got: %d", len(res))
	}

	if total, ok := res["total"]; !ok {
		t.Fatalf("Expected 'Values()' to have a 'total'")
	} else if total != 0 {
		t.Fatalf("Expected 'Values()' 'total' to be 0, got: %f", total)
	}

	acc = &Accumulator{
		outputType: "counter",
		ttl:        time.Hour,
		values: []*accValue{
			{1, now.Add(-3 * time.Hour)},
		},
	}

	res = acc.Values()

	if len(res) != 2 {
		t.Fatalf("Expected 'Values()' length to be 2, got: %d", len(res))
	}

	if total, ok := res["total"]; !ok {
		t.Fatalf("Expected 'Values()' to have a 'total'")
	} else if total != 0 {
		t.Fatalf("Expected 'Values()' 'total' to be 0, got: %f", total)
	}

	if sum, ok := res["sum"]; !ok {
		t.Fatalf("Expected 'Values()' to have a 'sum'")
	} else if sum != 0 {
		t.Fatalf("Expected 'Values()' 'sum' to be 0, got: %f", sum)
	}
}
